@@ -1,7 +1,6 @@
-#!/usr/bin/python
 #-*- coding: utf-8 -*-
 
-# Copyright 2012 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2012-2013 Calculate Ltd. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -23,9 +22,18 @@ from calculate.lib.cl_print import color_print
 from calculate.lib.cl_lang import setLocalTranslate
 setLocalTranslate('cl_core3',sys.modules[__name__])
 from itertools import *
+from api_types import ViewInfoAdapter, ArrayReturnedMessage, FieldAdapter
+from calculate.lib.utils.colortext import get_terminal_print, TextState
 
 colorPrint = color_print()
 
+class GotErrorField(Exception):
+    """
+    Исключение о получение поля error среди
+    элементов view
+    """
+    pass
+
 class BoolAction(argparse.Action):
     reTrue = re.compile("^(?:on)$",re.I)
     reFalse = re.compile("^(?:off)$",re.I)
@@ -166,17 +174,18 @@ def get_method_argparser(view, args, cl_core = False):
                 not y[2:].lower().startswith("off") \
                 else x+[y],args,[])
         return args
+    def get_list(data):
+        if data is not None:
+            for entry in data if cl_core else data[0]:
+                yield entry
+
     parser = argparse.ArgumentParser(prog=progr, add_help=False,
                                  formatter_class=RawAndDefaultsHelpFormatter)
     parser.fixBoolVariables = fixBoolVariables
 
-    if view.groups is None:
-        return parser
-    for Group in getlist(view.groups):
-        if not Group.fields:
-            continue
+    for Group in ifilter(lambda x:x.fields, get_list(view.groups)):
         group = parser.add_argument_group(Group.name)
-        for field in getlist(Group.fields):
+        for field in get_list(Group.fields):
             if field.element == 'error':
                 error_flag = True
                 colorPrint.printERROR(field.label)
@@ -227,6 +236,10 @@ def get_method_argparser(view, args, cl_core = False):
                                               else field.name.upper()
                     #if ':' in data['metavar']:
                     #    data['metavar'] = field.name.upper()
+                    if "choice" in field.type:
+                        data['help'] = "%s (%s)" % (
+                            data['help'],
+                            _("'list' for displaying possible values"))
                 if field.element in ['check']:
                     data['metavar'] = "ON/OFF"
                 try:
@@ -242,7 +255,7 @@ def get_method_argparser(view, args, cl_core = False):
         '-f', '--force', action='store_true', default=False,
         dest = 'no_questions', help=_('silent during the process'))
     if error_flag:
-        raise Exception
+        raise GotErrorField
     return parser
 
 def set_obj_item(client, param_object, field_name, value):
@@ -351,66 +364,101 @@ def set_table_pwd(client, param_object, field, value):
         setattr (param_object, field.name, result)
         return param_object
 
-def check_result_msg(method_result, view, input_error_dict = {}):
-    password_errors = {}
-    if hasattr (method_result, 'ReturnedMessage'):
-        method_result = method_result.ReturnedMessage
 
-    if hasattr (view.groups, 'GroupField'):
-        groups = view.groups.GroupField
+def display_error(error, args, groups):
+    params_text = ''
+    sys.stdout.write('\r')
+    sys.stdout.flush()
+    list_answer = False
+    varname, comments, values, value,\
+        list_value= None, None, None, None, []
+    if error.type != "commonerror":
+        for group in groups:
+            for field in group.fields:
+                if field.name == error.field:
+                    if not args is None:
+                        if (getattr(args, field.name) == "list" and
+                                    "choice" in field.type):
+                            if error.field_obj:
+                                list_answer = True
+                                varname = error.field_obj.label or \
+                                          error.field_obj.name
+                                comments = error.field_obj.comments
+                                values = error.field_obj.choice
+                                value = error.field_obj.value
+                                list_value = error.field_obj.listvalue or []
+                        params_text += getErrorOnParam(args, field)
+                    else:
+                        if field.opt.shortopt or field.opt.longopt:
+                            params_text += _('Wrong option ')
+                            params_text += ' ' + ', '.join(
+                                filter(None, [field.opt.shortopt,
+                                              field.opt.longopt])) + '. %s'
+    if list_answer:
+        _print = get_terminal_print(colorPrint.defaultPrint)
+        _print.foreground(TextState.Colors.WHITE)(
+            _("%s values:") % varname)
+        _print("\n")
+        #colorPrint.printSUCCESS("%s values:" % varname)
+        if comments and values:
+            maxlen = len(max(values, key=len))
+            for v,c in zip(values, comments):
+                if not v and not c:
+                    continue
+                _print("  ")
+                _print.bold("["+v+"]")
+                _print(" "*(maxlen-len(v)))
+                _print("  ")
+                _print(c)
+                if value == v or v in list_value:
+                    _print(" ")
+                    _print.bold.foreground(TextState.Colors.WHITE)("*")
+                _print("\n")
+        if not comments:
+            for v in filter(None,values):
+                _print("  ")
+                _print.bold("["+v+"]")
+                if value == v:
+                    _print(" ")
+                    _print.bold.foreground(TextState.Colors.WHITE)("*")
+                _print("\n")
+    elif error.type != "commonerror":
+        colorPrint.printERROR(params_text % error.message)
     else:
-        groups = view.groups
+        colorPrint.printWARNING(error.message)
+
+
+def check_result_msg(method_result, view, input_error_dict={}, args=None):
+    password_errors = {}
+    method_result = ArrayReturnedMessage.from_detect(method_result)
+    view = ViewInfoAdapter.from_detect(view)
+
     for error in method_result:
         if error.type == 'pwderror':
             password_errors[error.field] = error.message
             continue
 
-        params_text = ''
-        if error.type != "commonerror":
-            for Group in groups:
-                if hasattr (Group.fields, 'Field'):
-                    fields = Group.fields.Field
-                else:
-                    fields = Group.fields
-                for field in fields:
-                    if field.name == error.field:
-                        if field.opt.shortopt or field.opt.longopt:
-                            params_text += _('Wrong option ')
-                            params_text += ' '+', '.join(filter(None,
-                                [field.opt.shortopt, field.opt.longopt])) \
-                                + '. '
+        display_error(error, args, view.groups)
 
-        sys.stdout.write('\r')
-        sys.stdout.flush()
-        if error.type != "commonerror":
-            colorPrint.printERROR(params_text + error.message)
-        else:
-            colorPrint.printWARNING(params_text + error.message)
-
-    if len(password_errors) < len(method_result):
-        return None
-    else:
+    # если все ошибки связаны с паролем
+    if len(password_errors) == len(method_result):
         if not dict([x for x in input_error_dict.items() \
-                            if x not in password_errors.items()]) and \
-                            not dict([x for x in password_errors.items() \
-                            if x not in input_error_dict.items()]):
+                     if x not in password_errors.items()]) and \
+                not dict([x for x in password_errors.items() \
+                          if x not in input_error_dict.items()]):
             return None
         return password_errors
+    else:
+        return None
+
 
 def get_param_pwd(check_res, view, param_object, client = None,
                   stdin_passwd=False):
-    if hasattr (view.groups, 'GroupField'):
-        groups = view.groups.GroupField
-    else:
-        groups = view.groups
+    view = ViewInfoAdapter.from_detect(view)
     for pwd_field in check_res:
-        _print (check_res[pwd_field])
-        for Group in groups:
-            if hasattr (Group.fields, 'Field'):
-                fields = Group.fields.Field
-            else:
-                fields = Group.fields
-            for field in fields:
+        _print(check_res[pwd_field])
+        for group in view.groups:
+            for field in group.fields:
                 if field.name == pwd_field:
                     if field.element == 'table':
                         value = get_password(getfromstdin=stdin_passwd)
@@ -424,26 +472,16 @@ def get_param_pwd(check_res, view, param_object, client = None,
                         setattr(param_object, pwd_field, value)
     return param_object
 
-def collect_object(client, param_object, view, args,wait_thread = None,
-                   stdin_passwd = False):
+
+def collect_object(client, param_object, view, args, wait_thread=None,
+                   stdin_passwd=False):
     """
     Collect Info object by args
     """
     steps = None
-    if type(view.groups) == list:
-        # for local call method
-        groups = view.groups
-    else:
-        groups = view.groups[0]
-    for Group in groups:
-        if not Group.fields:
-            continue
-        if type(Group.fields) == list:
-            # for local call method
-            fields = Group.fields
-        else:
-            fields = Group.fields[0]
-        for field in fields:
+    view = ViewInfoAdapter.from_detect(view)
+    for group in view.groups:
+        for field in group.fields:
             if field.uncompatible:
                 continue
             if field.element in ['check', 'check_tristate'] and \
@@ -492,14 +530,16 @@ def collect_object(client, param_object, view, args,wait_thread = None,
                                             val)
 
             elif field.element in ['input', 'openfile',\
-                                   'file', 'radio']:
+                                   'file', 'radio','combo','comboEdit']:
                 param_object = set_obj_item(client, param_object, field.name,
                                             _getattr(args, field.name))
-            elif field.element in ['password'] and _getattr(args, field.name) \
+            elif 'passwd' in field.element and _getattr(args, field.name) \
                 or field.type and  "need" in field.type:
                 if wait_thread:
                     wait_thread.pause()
-                password=get_password("%s: "%field.label,_('Repeat password: '),
+                label = field.label or _("Password")
+                password=get_password("%s: "%label,
+                                      _('Repeat password: '),
                                       getfromstdin=stdin_passwd,
                                       needrepeat=not "one" in field.type)
                 if password is None:
@@ -508,21 +548,6 @@ def collect_object(client, param_object, view, args,wait_thread = None,
                                             password)
                 if wait_thread:
                     wait_thread.resume()
-            elif field.element in ['combo', 'comboEdit']:
-                value = _getattr(args, field.name)
-                if value:
-                    if hasattr (field.choice, 'string'):
-                        choice = field.choice.string
-                    else:
-                        choice = field.choice
-                    for item in choice:
-                        if item:
-                            if value.lower() == item.lower():
-                                value = item
-                                break
-
-                param_object = set_obj_item(client, param_object, field.name,
-                                            value)
             elif field.element in ['multichoice', 'multichoice_add', \
                                    'selecttable', 'selecttable_add']:
                 val = _getattr(args, field.name)
@@ -583,8 +608,6 @@ def collect_table(field, val_list, client, wait_thread = None,
     # standard syntax
     else:
         val_table = map(lambda x: x.split(':'), val_list)
-    if "DEBUG" in os.environ:
-        print "VAL_TABLE",val_table
 
     if type(field.tablevalue.values) in [list, tuple]:
         choiceValue = field.tablevalue.values
@@ -666,13 +689,10 @@ def collect_table(field, val_list, client, wait_thread = None,
     return listToArrayArray(client, obj_body)
 
 def collect_obj_body(body, field):
-    column = len(field.tablevalue.head) if type(field.tablevalue.head) == list\
-                                        else len(field.tablevalue.head.string)
+    field = FieldAdapter.from_detect(field)
+    column = len(field.tablevalue.head)
     result_table = []
-    if type(field.tablevalue.values) in [list, tuple]:
-        ChoiceValue = field.tablevalue.values
-    else:
-        ChoiceValue = field.tablevalue.values.ChoiceValue
+    ChoiceValue = field.tablevalue.values
     for i in range(len(body)):
         temp_row = []
         for j in range(column):
@@ -718,7 +738,7 @@ def getErrorOnParam(args,field):
     Get errors for param
     """
     params_text = ""
-    if any("-" in x 
+    if any("-" in x
         for x in filter(None,(field.opt.longopt,field.opt.shortopt))):
         paramName = ', '.join(filter(None,
                 [field.opt.shortopt, field.opt.longopt]))
@@ -726,14 +746,16 @@ def getErrorOnParam(args,field):
         paramName = field.opt.metavalue
     if getattr(args,field.name) is None and \
         not "need" in field.type:
-        params_text += _("Need to specify the parameter ")
+        params_text += "%s. " + _("Use the parameter") + " "
         params_text += paramName+'. '
     else:
         if "need" in field.type:
             params_text += \
-                _('Error in field \'%s\'. ')%field.label
+                _('Error in field \'%s\'. ')%field.label + " %s"
+        elif (getattr(args, field.name) == "list" and
+                        "choice" in field.type):
+            params_text += "%s"
         else:
             params_text += _('Error in parameter ')
-            params_text += paramName +'. '
+            params_text += paramName +'. %s'
     return params_text
-
